#!/usr/bin/env python
# coding=utf-8
import uuid

import pandas as pd

from .base import Tools, Html, Js


def tabulator_segment(plot):
    return f"""
        var lineFormatter = function(cell, formatterParams, onRendered){{
            onRendered(function(){{ 
                $(cell.getElement()).sparkline(cell.getValue(), {{width:"100%", type:"line"}});
            }});
        }};
        
        var barFormatter = function(cell, formatterParams, onRendered){{
            onRendered(function(){{ 
                $(cell.getElement()).sparkline(cell.getValue(), {{width:"100%", type:"bar"}});
            }});
        }};
        
        var tristateFormatter = function(cell, formatterParams, onRendered){{
            onRendered(function(){{ 
                $(cell.getElement()).sparkline(cell.getValue(), {{width:"100%", type:"tristate"}});
            }});
        }};
        
        var boxFormatter = function(cell, formatterParams, onRendered){{
            onRendered(function(){{
                $(cell.getElement()).sparkline(cell.getValue(), {{width:"100%", type:"box"}});
            }});
        }};
        var pieFormatter = function(cell, formatterParams, onRendered){{
            onRendered(function(){{
                $(cell.getElement()).sparkline(cell.getValue(), {{width:"100%", type:"pie"}});
            }});
        }};
        var bulletFormatter = function(cell, formatterParams, onRendered){{
            onRendered(function(){{
                $(cell.getElement()).sparkline(cell.getValue(), {{width:"100%", type:"bullet"}});
            }});
        }};
        var discreteFormatter = function(cell, formatterParams, onRendered){{
            onRendered(function(){{
                $(cell.getElement()).sparkline(cell.getValue(), {{width:"100%", type:"discrete"}});
            }});
        }};
        var tabledata = {plot.tabledata};
        var table = new Tabulator("#{plot.plot_id}", {{
            height:{plot.height},
            data:tabledata,
            layout:"fitColumns",
            columns:{plot.columns},
        }});
        """


class Tabulator(object):
    """
    g2plot
    """

    def __init__(self, df: pd.DataFrame, sparkline_dict=None, width_dict=None, formatter_dict=None,
                 height: str = "500"):
        """
        tabulator
        :param df: []
        :param sparkline_dict: {'col':'line/bar/tristate/discrete/bullet/pie/box'}
        :param width_dict: {'col':'120'}
        :param formatter_dict: {'col':'progress/star/tickCross/color'}
        :param height:500
        """
        self.tabledata = Tools.convert_dict_to_js(df.to_dict(orient='records'))
        cols = []
        for col in df.columns:
            column = {'title': col, 'field': col}
            if sparkline_dict is not None and col in sparkline_dict.keys():
                column['formatter'] = Js(sparkline_dict[col] + "Formatter")
            if width_dict is not None and col in width_dict.keys():
                column['width'] = width_dict[col]
            if formatter_dict is not None and col in formatter_dict.keys():
                column['formatter'] = formatter_dict[col]
                if formatter_dict[col] == 'progress':
                    column['formatterParams'] = {"color": ["green", "orange", "red"]}
            cols.append(column)
        self.columns = Tools.convert_dict_to_js(cols)
        self.height = height
        self.plot_id = "u" + uuid.uuid4().hex
        self.width = "100%"

    @staticmethod
    def reduce_dataframe(df: pd.DataFrame, reduce_axis='index'):
        """
        :param df: 示例 pd.DataFrame(index=['2021-01-01','2021-01-02'],columns=['000001.SZ','000002.SZ'])
        :param reduce_axis: index按索引方向合并值到数组，columns 按列方向合并值到数组
        :return:
        """
        return df.apply(lambda s: s.tolist(), axis=reduce_axis, result_type='reduce')

    def render_notebook(self) -> Html:
        """
        在jupyter notebook 环境输出
        :return:
        """
        plot = self
        html = f"""
        <script>
          requirejs.config(
                     {{paths: {{ 
                        'tabulator': ['https://cdn.staticfile.org/tabulator/5.2.3/js/tabulator.min'],
                        'jquery':['https://cdn.staticfile.org/jquery/3.6.0/jquery.min'],
                        'sparkline':['https://cdn.staticfile.org/jquery-sparklines/2.1.2/jquery.sparkline.min'],
                     }},}}
                );
        </script>
        <style>
          #{plot.plot_id} {{
            width:{plot.width};
            height:{plot.height};
         }}
         .jqstooltip {{
          -webkit-box-sizing: content-box;
          -moz-box-sizing: content-box;
          box-sizing: content-box;
        }}
        </style>
        <div id="{plot.plot_id}"></div>
        <script>
         require(['tabulator','jquery','sparkline'],function(Tabulator,$,sparkline) {{
                var element = document.createElement("link");
                        element.setAttribute("rel", "stylesheet");
                        element.setAttribute("type", "text/css");
                        element.setAttribute("href", "https://cdn.staticfile.org/tabulator/5.2.3/css/tabulator.min.css");
                        document.getElementsByTagName("head")[0].appendChild(element);
                window.Tabulator=tabulator;
                """ + tabulator_segment(plot) + f"""
            }});
        </script>

        """
        return Html(html)

    def render_jupyterlab(self) -> Html:
        """
        在jupyterlab 环境输出
        :return:
        """
        plot = self
        html = f"""
            <style>
             #{plot.plot_id} {{
                width:{plot.width};
                height:{plot.height};
             }}
             .jqstooltip {{
              -webkit-box-sizing: content-box;
              -moz-box-sizing: content-box;
              box-sizing: content-box;
            }}
            </style>
            <div id="{plot.plot_id}"></div>
            <script>
            // load javascript
            
            new Promise(function(resolve, reject) {{
              var script = document.createElement("script");
              script.onload = resolve;
              script.onerror = reject;
              script.src = "https://cdn.staticfile.org/tabulator/5.2.3/js/tabulator.min.js";
              document.head.appendChild(script);
                var jq = document.createElement("script");
              script.onload = resolve;
              script.onerror = reject;
              script.src = "https://cdn.staticfile.org/jquery/3.6.0/jquery.min.js";
              document.head.appendChild(jq);
                var sparkline = document.createElement("script");
              script.onload = resolve;
              script.onerror = reject;
              script.src = "https://cdn.staticfile.org/jquery-sparklines/2.1.2/jquery.sparkline.min.js";
              document.head.appendChild(sparkline);
                var element = document.createElement("link");
                element.setAttribute("rel", "stylesheet");
                element.setAttribute("type", "text/css");
                element.setAttribute("href", "https://cdn.staticfile.org/tabulator/5.2.3/css/tabulator.min.css");
                document.getElementsByTagName("head")[0].appendChild(element);
            }}).then(() => {{
              """ + tabulator_segment(plot) + f"""
            }});
            </script>
            """
        return Html(html)

    def render_html(self) -> str:
        """
        渲染html字符串，可以用于 streamlit
        :return:
        """
        plot = self
        html = f"""
            <!DOCTYPE html>
            <html>
            <head>
              <meta charset="UTF-8">
              <title></title>
                <style>
                  #{plot.plot_id} {{
                        width:{plot.width};
                        height:{plot.height};
                     }}
                    .jqstooltip {{
              -webkit-box-sizing: content-box;
              -moz-box-sizing: content-box;
              box-sizing: content-box;
            }}
                </style>
                <script type="text/javascript" src="https://cdn.staticfile.org/tabulator/5.2.3/js/tabulator.min.js"></script>
             <script type="text/javascript" src="https://cdn.staticfile.org/jquery/3.6.0/jquery.min.js"></script>
             <script type="text/javascript" src="https://cdn.staticfile.org/jquery-sparklines/2.1.2/jquery.sparkline.min.js"></script>
            <link rel='stylesheet' href='https://cdn.staticfile.org/tabulator/5.2.3/css/tabulator.min.css'>
            </head>
            <body>
              <div id="{plot.plot_id}" ></div>
              <script>
                 {plot.extra_js}
            """ + tabulator_segment(plot) + f"""
            
              </script>
            </body>
            </html>
            """
        return html

    def render_html_fragment(self):
        """
        渲染html 片段，方便一个网页输出多个图表
        :return:
        """
        plot = self
        html = f"""
            <div>
             <script type="text/javascript" src="https://cdn.staticfile.org/tabulator/5.2.3/js/tabulator.min.js"></script>
             <script type="text/javascript" src="https://cdn.staticfile.org/jquery/3.6.0/jquery.min.js"></script>
             <script type="text/javascript" src="https://cdn.staticfile.org/jquery-sparklines/2.1.2/jquery.sparkline.min.js"></script>
            <link rel='stylesheet' href='https://cdn.staticfile.org/tabulator/5.2.3/css/tabulator.min.css'>
             <style>
                  #{plot.plot_id} {{
                        width:{plot.width};
                        height:{plot.height};
                     }}
                     .jqstooltip {{
              -webkit-box-sizing: content-box;
              -moz-box-sizing: content-box;
              box-sizing: content-box;
            }}
             </style>
             <div id="{plot.plot_id}" ></div>
              <script>
            """ + tabulator_segment(plot) + f"""
              </script>
            </div>
            """
        return html

    def _repr_html_(self):
        """
        jupyter 环境，直接输出
        :return:
        """
        plot = self
        html = f"""

        <style>
          #{plot.plot_id} {{
            width:{plot.width};
            height:{plot.height};
         }}
         .jqstooltip {{
          -webkit-box-sizing: content-box;
          -moz-box-sizing: content-box;
          box-sizing: content-box;
        }}
        </style>
        <div id="{plot.plot_id}"></div>
        <script>
          if (typeof require !== 'undefined'){{
                requirejs.config(
                     {{paths: {{ 
                        'tabulator': ['https://cdn.staticfile.org/tabulator/5.2.3/js/tabulator.min'],
                        'jquery':['https://cdn.staticfile.org/jquery/3.6.0/jquery.min'],
                        'sparkline':['https://cdn.staticfile.org/jquery-sparklines/2.1.2/jquery.sparkline.min'],
                     }},}}
                );
                require(['tabulator','jquery','sparkline'],function(Tabulator,$,sparkline) {{

                       var element = document.createElement("link");
                        element.setAttribute("rel", "stylesheet");
                        element.setAttribute("type", "text/css");
                        element.setAttribute("href", "https://cdn.staticfile.org/tabulator/5.2.3/css/tabulator.min.css");
                        document.getElementsByTagName("head")[0].appendChild(element);
                        """ + tabulator_segment(plot) + f"""
                    }});
             }}else{{
               new Promise(function(resolve, reject) {{
                  var script = document.createElement("script");
                  script.onload = resolve;
                  script.onerror = reject;
                  script.src = "https://cdn.staticfile.org/tabulator/5.2.3/js/tabulator.min.js";
                  document.head.appendChild(script);
                    var jq = document.createElement("script");
                  script.onload = resolve;
                  script.onerror = reject;
                  script.src = "https://cdn.staticfile.org/jquery/3.6.0/jquery.min.js";
                  document.head.appendChild(jq);
                    var sparkline = document.createElement("script");
                  script.onload = resolve;
                  script.onerror = reject;
                  script.src = "https://cdn.staticfile.org/jquery-sparklines/2.1.2/jquery.sparkline.min.js";
                  document.head.appendChild(sparkline);
                    var element = document.createElement("link");
                    element.setAttribute("rel", "stylesheet");
                    element.setAttribute("type", "text/css");
                    element.setAttribute("href", "https://cdn.staticfile.org/tabulator/5.2.3/css/tabulator.min.css");
                    document.getElementsByTagName("head")[0].appendChild(element);
                }}).then(() => {{
                  """ + tabulator_segment(plot) + f"""
                }});
             }}

        </script>
        """
        return Html(html).data



#!/usr/bin/env python
# coding=utf-8
import copy
import uuid

from .base import Tools, Html


class HighCharts(object):
    """
    echarts
    """

    def __init__(self, options: dict = None, extra_js: str = "", width: str = "100%",
                 height: str = "500px"):
        """
        :param options: python词典类型的echarts option
        :param extra_js: 复杂图表需要声明定义额外js函数的，通过这个字段传递
        :param width: 输出div的宽度 支持像素和百分比 比如800px/100%
        :param height: 输出div的高度 支持像素和百分比 比如800px/100%
        """
        self.options = options
        self.js_options = ""
        self.width = width
        self.height = height
        self.plot_id = "u" + uuid.uuid4().hex
        self.extra_js = extra_js

    def print_options(self, drop_data=False):
        """
        格式化打印options 方便二次修改
        :param drop_data: 是否过滤掉data，减小打印长度，方便粘贴
        :return:
        """
        dict_options = copy.deepcopy(self.options)
        if drop_data:
            series_count = len(dict_options['series'])
            for i in range(0, series_count):
                dict_options['series'][i]['data'] = []
        Tools.convert_js_to_dict(Tools.convert_dict_to_js(dict_options), print_dict=True)

    def dump_options(self):
        """
         导出 js option字符串表示
        :return:
        """
        self.js_options = Tools.convert_dict_to_js(self.options)
        return self.js_options

    def render_notebook(self) -> Html:
        """
        在jupyter notebook 环境输出
        :return:
        """
        self.js_options = Tools.convert_dict_to_js(self.options)
        plot = self
        html = f"""
        <style>
          #{plot.plot_id} {{
            width:{plot.width};
            height:{plot.height};
         }}
        </style>
        <div id="{plot.plot_id}"></div>
        <script>
            require.config({{
                    packages: [{{
                        name: 'highcharts',
                        main: 'highcharts'
                    }}],
                    paths: {{
                        'highcharts': 'https://code.highcharts.com'
                    }}
               }});
              require(['highcharts','highcharts/modules/streamgraph','highcharts/modules/arc-diagram','highcharts/modules/sankey','highcharts/modules/dependency-wheel'], function (Highcharts) {{
                {plot.extra_js}
                var options_{plot.plot_id} = {plot.js_options};
                Highcharts.chart('{plot.plot_id}',options_{plot.plot_id})
              }});
        </script>
        """
        return Html(html)

    def render_jupyterlab(self) -> Html:
        """
        在jupyterlab 环境输出
        :return:
        """
        self.js_options = Tools.convert_dict_to_js(self.options)
        plot = self
        html = f"""
            <style>
             #{plot.plot_id} {{
                width:{plot.width};
                height:{plot.height};
             }}
            </style>
            <div id="{plot.plot_id}"></div>
              <script>
                // load javascript
                new Promise(function(resolve, reject) {{
                  var script = document.createElement("script");
                  script.onload = resolve;
                  script.onerror = reject;
                  script.src = "https://code.highcharts.com/highcharts.js";
                  document.head.appendChild(script);
                  var scriptSG = document.createElement("script");
                      scriptSG.onload = resolve;
                      scriptSG.onerror = reject;
                      scriptSG.src = "https://code.highcharts.com/modules/streamgraph.js";
                      document.head.appendChild(scriptSG);
                  var wheel = document.createElement("script");
                      scriptSG.onload = resolve;
                      scriptSG.onerror = reject;
                      scriptSG.src = "https://code.highcharts.com/modules/dependency-wheel.js";
                      document.head.appendChild(wheel);
                  var sankey = document.createElement("script");
                      scriptSG.onload = resolve;
                      scriptSG.onerror = reject;
                      scriptSG.src = "https://code.highcharts.com/modules/sankey.js";
                      document.head.appendChild(sankey);
                  var arc_diagram = document.createElement("script");
                      scriptSG.onload = resolve;
                      scriptSG.onerror = reject;
                      scriptSG.src = "https://code.highcharts.com/modules/arc-diagram.js";
                      document.head.appendChild(arc_diagram);
                }}).then(() => {{
                   {plot.extra_js}
                   var options_{plot.plot_id} = {plot.js_options};
                   Highcharts.chart('{plot.plot_id}',options_{plot.plot_id})
                }});
              </script>
            """
        return Html(html)

    def render_html(self) -> str:
        """
        渲染html字符串，可以用于 streamlit
        :return:
        """
        self.js_options = Tools.convert_dict_to_js(self.options)
        plot = self
        html = f"""
        <!DOCTYPE html>
        <html>
        <head>
          <meta charset="UTF-8">
          <title></title>
            <style>
              #{plot.plot_id} {{
                    width:{plot.width};
                    height:{plot.height};
                 }}
            </style>
        <script src="https://code.highcharts.com/highcharts.js"></script>
        <script src="https://code.highcharts.com/modules/streamgraph.js"></script>
        <script src="https://code.highcharts.com/modules/dependency-wheel.js"></script>
         <script src="https://code.highcharts.com/modules/arc-diagram.js"></script>
         <script src="https://code.highcharts.com/modules/sankey.js"></script>
        </head>
        <body>
          <div id="{plot.plot_id}" ></div>
          <script>
            {plot.extra_js}
            var options_{plot.plot_id} = {plot.js_options};
            Highcharts.chart('{plot.plot_id}',options_{plot.plot_id})
          </script>
        </body>
        </html>
        """
        return html

    def render_html_fragment(self):
        """
        渲染html 片段，方便一个网页输出多个图表
        :return:
        """
        self.js_options = Tools.convert_dict_to_js(self.options)
        plot = self
        html = f"""
        <div>
        <script src="https://code.highcharts.com/highcharts.js"></script>
        <script src="https://code.highcharts.com/modules/streamgraph.js"></script>
        <script src="https://code.highcharts.com/modules/dependency-wheel.js"></script>
         <script src="https://code.highcharts.com/modules/arc-diagram.js"></script>
         <script src="https://code.highcharts.com/modules/sankey.js"></script>
         <style>
              #{plot.plot_id} {{
                    width:{plot.width};
                    height:{plot.height};
              }}
         </style>
         <div id="{plot.plot_id}" ></div>
          <script>
            {plot.extra_js}
            var options_{plot.plot_id} = {plot.js_options};
            Highcharts.chart('{plot.plot_id}',options_{plot.plot_id})
          </script>
        </div>
        """
        return html

    def _repr_html_(self):
        """
        jupyter 环境，直接输出
        :return:
        """
        self.js_options = Tools.convert_dict_to_js(self.options)
        plot = self
        html = f"""
        <script>
        
        </script>
        <style>
          #{plot.plot_id} {{
            width:{plot.width};
            height:{plot.height};
         }}
        </style>
        <div id="{plot.plot_id}"></div>
        <script >
          {plot.extra_js}
          var options_{plot.plot_id} = {plot.js_options};
        </script>
        <script>
          if (typeof require !== 'undefined'){{
              require.config({{
                    packages: [{{
                        name: 'highcharts',
                        main: 'highcharts'
                    }}],
                    paths: {{
                        'highcharts': 'https://code.highcharts.com'
                    }}
               }});
              require(['highcharts','highcharts/modules/streamgraph','highcharts/modules/arc-diagram','highcharts/modules/sankey','highcharts/modules/dependency-wheel'], function (Highcharts) {{
                Highcharts.chart('{plot.plot_id}',options_{plot.plot_id})
              }});
          }}else{{
            new Promise(function(resolve, reject) {{
                 var script = document.createElement("script");
              script.onload = resolve;
              script.onerror = reject;
              script.src = "https://code.highcharts.com/highcharts.js";
              document.head.appendChild(script);
              var scriptSG = document.createElement("script");
                  scriptSG.onload = resolve;
                  scriptSG.onerror = reject;
                  scriptSG.src = "https://code.highcharts.com/modules/streamgraph.js";
                  document.head.appendChild(scriptSG);
              var wheel = document.createElement("script");
                  scriptSG.onload = resolve;
                  scriptSG.onerror = reject;
                  scriptSG.src = "https://code.highcharts.com/modules/dependency-wheel.js";
                  document.head.appendChild(wheel);
              var sankey = document.createElement("script");
                  scriptSG.onload = resolve;
                  scriptSG.onerror = reject;
                  scriptSG.src = "https://code.highcharts.com/modules/sankey.js";
                  document.head.appendChild(sankey);
              var arc_diagram = document.createElement("script");
                  scriptSG.onload = resolve;
                  scriptSG.onerror = reject;
                  scriptSG.src = "https://code.highcharts.com/modules/arc-diagram.js";
                  document.head.appendChild(arc_diagram);
            
            }}).then(() => {{
                {plot.extra_js}
               Highcharts.chart('{plot.plot_id}',options_{plot.plot_id})
            }});
          }}
        </script>
        """
        return Html(html).data



#!/usr/bin/env python
# coding=utf-8
import uuid

import pandas as pd

from .base import Tools, Html

KlineCharts_JS_URL: str = "https://cdn.jsdelivr.net/npm/klinecharts@latest/dist/klinecharts.min.js"


# language=jinja2

def kline_chart_segment(plot):
    parts = []
    parts.append(
        f"""var chart_{plot.plot_id} = klinecharts.init("{plot.plot_id}",
        {{
            grid: {{ 
                show: true, horizontal: {{ show: true, size: 2, color: '#CFCFCF', style: 'dash'}},
                vertical: {{ show: true, size: 2, color: '#CFCFCF',  style: 'dash'}},
                'candle':{{'bar':{{'upColor':'#EF5350','downColor':'#26A69A'}} }},
                'technicalIndicator':{{
                    'bar':{{'upColor':'#EF5350','downColor':'#26A69A'}} 
                 }}
            }}
         }}
         );""")
    for bt in plot.bottom_indicators:
        parts.append(f"""var btm_{bt}_{plot.plot_id} = chart_{plot.plot_id}.createTechnicalIndicator('{bt}', false)""")
    for mi in plot.main_indicators:
        parts.append(f"""chart_{plot.plot_id}.createTechnicalIndicator('{mi}', true,{{id:"candle_pane"}})""")
    if len(plot.mas) > 0:
        parts.append(
            f"""chart_{plot.plot_id}.overrideTechnicalIndicator({{name: 'MA',calcParams: {str(plot.mas)} }},"candle_pane")""")
    if len(plot.segments) > 0:
        for seg in plot.segments:
            parts.append(f"""
            chart_{plot.plot_id}.createShape({{name: 'segment',points:[{{timestamp:{seg['start_time']},value:{seg['start_price']}}},{{timestamp:{seg['end_time']},value:{seg['end_price']}}}]}},"candle_pane")
            """)
    parts.append(f"""chart_{plot.plot_id}.applyNewData(data_{plot.plot_id})""")
    return "\n".join(parts)


class KlineCharts(object):
    """
    g2plot
    """

    def __init__(self, df: pd.DataFrame, mas=[5, 10, 30, 60, 120, 250], main_indicators=["MA"],
                 bottom_indicators=["VOL", "MACD"], df_segments: pd.DataFrame = None,
                 extra_js: str = "", width: str = "100%",
                 height: str = "500px"):
        """
        k线图
        :param df: [open,high,low,close,volume,turnover,timestamp]
        :param mas: [5, 10, 30, 60, 120, 250]
        :param main_indicators: 主图显示的指标列表 MA,EMA,SMA,BOLL,SAR,BBI
        :param bottom_indicators:副图显示指标列表 VOL,MACD,KDJ,RSI,BIAS,BBAR,CCI,DMI,CR,PSY,DMA,TRIX,OBV,VR,WR,MTM,EMV,SAR,SMA,ROC,PVT,BBI,AO
        :param df_segments:[start_time,start_price,end_time,end_price]
        :param extra_js:
        :param width:
        :param height:
        """
        data = df.copy()
        data['timestamp'] = (pd.to_datetime(data['timestamp']) - pd.Timedelta(hours=8)).view("i8") // 10 ** 6
        data = data.sort_values(by=['timestamp'])
        if len(mas) > 0 and "MA" not in main_indicators:
            main_indicators.append("MA")
        self.data = Tools.convert_dict_to_js(data.to_dict(orient='records'))
        if df_segments is not None:
            df_seg = df_segments.copy()
            df_seg['start_time'] = (pd.to_datetime(df_seg['start_time']) - pd.Timedelta(hours=8)).view(
                "i8") // 10 ** 6
            df_seg['end_time'] = (pd.to_datetime(df_seg['end_time']) - pd.Timedelta(hours=8)).view("i8") // 10 ** 6
            self.segments = df_seg.to_dict(orient='records')
        else:
            self.segments = []
        self.mas = mas
        self.main_indicators = main_indicators
        self.bottom_indicators = bottom_indicators
        self.width = width
        self.height = height
        self.plot_id = "u" + uuid.uuid4().hex
        self.js_url = KlineCharts_JS_URL
        self.extra_js = extra_js

    def render_notebook(self) -> Html:
        """
        在jupyter notebook 环境输出
        :return:
        """
        plot = self

        html = f"""
        <script>
          require.config({{
            paths: {{
              "klinecharts": "{plot.js_url[:-3]}"
            }}
          }});
        </script>
        <style>
          #{plot.plot_id} {{
            width:{plot.width};
            height:{plot.height};
         }}
        </style>
        <div id="{plot.plot_id}"></div>
        <script>
          {plot.extra_js}
          var data_{plot.plot_id} = {plot.data}
          require(['klinecharts'], function (klinecharts) {{
            """ + kline_chart_segment(plot) + f"""
          }});
        </script>
        
        """
        return Html(html)

    def render_jupyterlab(self) -> Html:
        """
        在jupyterlab 环境输出
        :return:
        """
        plot = self
        html = f"""
            <style>
             #{plot.plot_id} {{
                width:{plot.width};
                height:{plot.height};
             }}
            </style>
            <div id="{plot.plot_id}"></div>
            <script>
            // load javascript
            
            {plot.extra_js}
            new Promise(function(resolve, reject) {{
              var script = document.createElement("script");
              script.onload = resolve;
              script.onerror = reject;
              script.src = "{plot.js_url}";
              document.head.appendChild(script);
            }}).then(() => {{
              """ + kline_chart_segment(plot) + f"""
            }});
            </script>
            """
        return Html(html)

    def render_html(self) -> str:
        """
        渲染html字符串，可以用于 streamlit
        :return:
        """
        plot = self
        html = f"""
        <!DOCTYPE html>
        <html>
        <head>
          <meta charset="UTF-8">
          <title></title>
            <style>
              #{plot.plot_id} {{
                    width:{plot.width};
                    height:{plot.height};
                 }}
            </style>
           <script type="text/javascript" src="{plot.js_url}"></script>
        </head>
        <body>
          <div id="{plot.plot_id}" ></div>
          <script>
             {plot.extra_js}
        """ + kline_chart_segment(plot) + f"""

                  </script>
                </body>
                </html>
        """
        return html

    def render_html_fragment(self):
        """
        渲染html 片段，方便一个网页输出多个图表
        :return:
        """
        plot = self
        html = f"""
        <div>
         <script type="text/javascript" src="{plot.js_url}"></script>
         <style>
              #{plot.plot_id} {{
                    width:{plot.width};
                    height:{plot.height};
              }}
         </style>
         <div id="{plot.plot_id}" ></div>
          <script>
            {plot.extra_js}
        """ + kline_chart_segment(plot) + f"""
          </script>
        </div>
        """
        return html

    def _repr_html_(self):
        """
        jupyter 环境，直接输出
        :return:
        """
        plot = self
        html = f"""
        <style>
          #{plot.plot_id} {{
            width:{plot.width};
            height:{plot.height};
         }}
        </style>
        <div id="{plot.plot_id}"></div>
        <script>
          {plot.extra_js}
          var data_{plot.plot_id} = {plot.data}
          if (typeof require !== 'undefined'){{
              require.config({{
                paths: {{
                  "klinecharts": "{plot.js_url[:-3]}"
                }}
              }});
              require(['klinecharts'], function (klinecharts) {{
                """ + kline_chart_segment(plot) + f"""
             }});
             }}else{{
               new Promise(function(resolve, reject) {{
                 var script = document.createElement("script");
                 script.onload = resolve;
                 script.onerror = reject;
                 script.src = "{plot.js_url}";
                 document.head.appendChild(script);
               }}).then(() => {{
                 """ + kline_chart_segment(plot) + f"""
               }});
             }}
        
        </script>
        """
        return Html(html).data
